INSERT INTO `cpanstats` (id,guid,state,postdate,tester,dist,version,platform,perl,osname,osvers,fulldate,type) VALUES (33578672,'0cbce1be-07f0-11e3-9db1-878205732d18','pass','201308','srezic@iconmobile.net (Slaven Rezic)','CPAN-Testers-WWW-Statistics','1.06','i386-freebsd','5.8.9','freebsd','8.0-release','201308181022',2);
INSERT INTO `cpanstats` (id,guid,state,postdate,tester,dist,version,platform,perl,osname,osvers,fulldate,type) VALUES (33578876,'5ad79194-6cdc-1014-b4e3-38f2223f278b','na','201308','mithaldu@yahoo.de (\"Christian Walde\")','CPAN-Testers-WWW-Statistics','1.06','MSWin32-x86-multi-thread','5.16.0','MSWin32','5.2','201308181034',2);
INSERT INTO `cpanstats` (id,guid,state,postdate,tester,dist,version,platform,perl,osname,osvers,fulldate,type) VALUES (33579060,'66261f56-07f3-11e3-ac5a-df7f19362e69','pass','201308','srezic@iconmobile.net (Slaven Rezic)','CPAN-Testers-WWW-Statistics','1.06','i386-freebsd','5.12.3','freebsd','8.0-release','201308181046',2);
INSERT INTO `cpanstats` (id,guid,state,postdate,tester,dist,version,platform,perl,osname,osvers,fulldate,type) VALUES (33578988,'b35d7c0c-07f2-11e3-afab-ccf1a108922b','pass','201308','srezic@iconmobile.net (Slaven Rezic)','CPAN-Testers-WWW-Statistics','1.06','i386-freebsd','5.14.0','freebsd','8.0-release','201308181041',2);
INSERT INTO `cpanstats` (id,guid,state,postdate,tester,dist,version,platform,perl,osname,osvers,fulldate,type) VALUES (33578912,'ffdb8ffc-07f1-11e3-a0a5-bfc8c70fa3a9','pass','201308','srezic@iconmobile.net (Slaven Rezic)','CPAN-Testers-WWW-Statistics','1.06','i386-freebsd','5.16.0 RC0','freebsd','8.0-release-p4','201308181036',2);
